@@ -49,11 +49,20 @@ pub struct LineStart<'a> {
     tab_start: usize,
     ix: usize,
     spaces_remaining: usize,
+    // no thematic breaks can occur before this offset.
+    // this prevents scanning over and over up to a certain point
+    min_hrule_offset: usize,
 }
 
 impl<'a> LineStart<'a> {
     pub fn new(text: &str) -> LineStart {
-        LineStart { text, tab_start: 0, ix: 0, spaces_remaining: 0 }
+        LineStart {
+            text,
+            tab_start: 0,
+            ix: 0,
+            spaces_remaining: 0,
+            min_hrule_offset: 0,
+        }
     }
 
     /// Try to scan a number of spaces.
@@ -151,9 +160,14 @@ impl<'a> LineStart<'a> {
         if self.ix < self.text.len() {
             let c = self.text.as_bytes()[self.ix];
             if c == b'-' || c == b'+' || c == b'*' {
-                if scan_hrule(&self.text[self.ix..]) > 0 {
-                    *self = save;
-                    return None;
+                if self.ix >= self.min_hrule_offset {
+                    // there could be an hrule here
+                    if let Err(min_offset) = scan_hrule(&self.text[self.ix..]) {
+                        self.min_hrule_offset = min_offset;
+                    } else {
+                        *self = save;
+                        return None;
+                    }
                 }
                 self.ix += 1;
                 if self.scan_space(1) || self.is_at_eol() {
@@ -318,24 +332,19 @@ pub fn scan_attr_value_chars(data: &str) -> usize {
     scan_while(data, is_valid_unquoted_attr_value_char)
 }
 
-// Maybe returning Option<usize> would be more Rustic?
-pub fn scan_eol(s: &str) -> (usize, bool) {
-    if s.is_empty() { return (0, true); }
+pub fn scan_eol(s: &str) -> Option<usize> {
+    if s.is_empty() { return Some(0); }
     let bytes = s.as_bytes();
     match bytes[0] {
-        b'\n' => (1, true),
-        b'\r' => (if s[1..].starts_with('\n') { 2 } else { 1 }, true),
-        _ => (0, false)
+        b'\n' => Some(1),
+        b'\r' => Some(if s[1..].starts_with('\n') { 2 } else { 1 }),
+        _ => None
     }
 }
 
 pub fn scan_blank_line(text: &str) -> Option<usize> {
     let i = scan_whitespace_no_nl(text);
-    if let (n, true) = scan_eol(&text[i..]) {
-        Some(i + n)
-    } else {
-        None
-    }
+    scan_eol(&text[i..]).map(|n| i + n)
 }
 
 pub fn scan_nextline(s: &str) -> usize {
@@ -352,7 +361,7 @@ pub fn scan_closing_code_fence(text: &str, fence_char: u8, n_fence_char: usize)
     i += num_fence_chars_found;
     let num_trailing_spaces = scan_ch_repeat(&text[i..], b' ');
     i += num_trailing_spaces;
-    if scan_eol(&text[i..]).1 { Some(i) } else { None }
+    scan_eol(&text[i..]).map(|_| i)
 }
 
 // returned pair is (number of bytes, number of spaces)
@@ -377,29 +386,34 @@ pub fn calc_indent(text: &str, max: usize) -> (usize, usize) {
     (i, spaces)
 }
 
-// return size of line containing hrule, including trailing newline, or 0
-// TODO: switch to Option return type
-pub fn scan_hrule(data: &str) -> usize {
+/// Scan hrule opening sequence.
+///
+/// Returns Ok(x) when it finds an hrule, where x is the 
+/// size of line containing the hrule, including the trailing newline.
+/// 
+/// Returns Err(x) when it does not find an hrule and x is
+/// the offset in data before no hrule can appear.
+pub fn scan_hrule(data: &str) -> Result<usize, usize> {
     let bytes = data.as_bytes();
     let size = data.len();
     let mut i = 0;
-    if i + 2 >= size { return 0; }
+    if i + 2 >= size { return Err(0); }
     let c = bytes[i];
-    if !(c == b'*' || c == b'-' || c == b'_') { return 0; }
+    if !(c == b'*' || c == b'-' || c == b'_') { return Err(i); }
     let mut n = 0;
     while i < size {
         match bytes[i] {
             b'\n' | b'\r' => {
-                i += scan_eol(&data[i..]).0;
+                i += scan_eol(&data[i..]).unwrap_or(0);
                 break;
             }
             c2 if c2 == c => n += 1,
             b' ' | b'\t' => (),
-            _ => return 0
+            _ => return Err(i)
         }
         i += 1;
     }
-    if n >= 3 { i } else { 0 }
+    if n >= 3 { Ok(i) } else { Err(i) }
 }
 
 /// Scan an ATX heading opening sequence.
@@ -420,7 +434,7 @@ pub fn scan_atx_heading(data: &str) -> Option<(usize, i32)> {
     } else {
         None
     }
-}
+    }
 
 /// Scan a setext heading underline.
 ///
@@ -455,9 +469,8 @@ pub fn scan_table_head(data: &str) -> (usize, Vec<Alignment>) {
         i += 1;
     }
     for c in data.as_bytes()[i..].iter() {
-        let eol = scan_eol(&data[i..]);
-        if eol.1 {
-            i += eol.0;
+        if let Some(n) = scan_eol(&data[i..]) {
+            i += n;
             break;
         }
         match *c {
@@ -496,34 +509,33 @@ pub fn scan_table_head(data: &str) -> (usize, Vec<Alignment>) {
     (i, cols)
 }
 
-// TODO: change return type to Option.
-// returns: number of bytes scanned, char
-pub fn scan_code_fence(data: &str) -> (usize, u8) {
+/// Scan code fence.
+///
+/// Returns number of bytes scanned and the char that is repeated to make the code fence.
+pub fn scan_code_fence(data: &str) -> Option<(usize, u8)> {
     if data.is_empty() {
-        return (0, 0);
+        return None;
     }
     let c = data.as_bytes()[0];
-    if !(c == b'`' || c == b'~') { return (0, 0); }
+    if !(c == b'`' || c == b'~') { return None; }
     let i = 1 + scan_ch_repeat(&data[1 ..], c);
     if i >= 3 {
         if c == b'`' {
             let next_line = i + scan_nextline(&data[i..]);
             if data[i..next_line].find('`').is_some() {
-                return (0, 0);
+                return None;
             }
         }
-        return (i, c);
+        return Some((i, c));
     }
-    (0, 0)
+    None
 }
 
-pub fn scan_blockquote_start(data: &str) -> usize {
-    if data.starts_with('>') {
-        let n = 1;
-        n + scan_ch(&data[n..], b' ')
-    } else {
-        0
+pub fn scan_blockquote_start(data: &str) -> Option<usize> {
+    if !data.starts_with('>') {
+        return None;
     }
+    Some(scan_ch(&data[1..], b' ') + 1)
 }
 
 /// This already assumes the list item has been scanned.
@@ -563,7 +575,9 @@ pub fn scan_listitem(data: &str) -> (usize, u8, usize, usize) {
     // TODO: replace calc_indent with scan_leading_whitespace, for tab correctness
     let (mut postn, mut postindent) = calc_indent(&data[w.. ], 5);
     if postindent == 0 {
-        if !scan_eol(&data[w..]).1 { return (0, 0, 0, 0); }
+        if scan_eol(&data[w..]).is_none() {
+            return (0, 0, 0, 0);
+        }
         postindent += 1;
     } else if postindent > 4 {
         postn = 1;
@@ -783,10 +797,9 @@ pub fn scan_html_type_7(data: &str) -> Option<usize> {
         loop {
             let whitespace = scan_whitespace_no_nl(&data[i..]);
             i += whitespace;
-            let c = data.as_bytes()[i];
+            let c = data.as_bytes().get(i);
             match c {
-                b'/' | b'>' => { 
-                    break; },
+                Some(b'/') | Some(b'>') => break,
                 _ => {},
             }
             if whitespace == 0 { return None; }
